/**************************************************************
*** Uninstalls APPX Connector service process for Windows, ****
************ Linux, and Mac operating systems *****************
**************************************************************/

/*
###############################################################################
###############################################################################
@usage: node appxConnector-uninstall [appxport] [mongoport]
@example 1: node appxConnector-uninstall appx 3014
@example 2: node appxConnector-uninstall mongo 3015
@example 3: node appxConnector-uninstall
###############################################################################
@Code defaults: if only 1 port is provided we will default port for
appxConnector. If no first argument is provided we will default to install both
connectors.
###############################################################################
###############################################################################
*/

var serverConnectorUnInstallVersionStr = "5.4.5.16110211";
var serverConnectorUnInstallVersionNum = 50405.16110211;

//Detects OS and sets the Service variable accordingly.
switch(process.platform){
	case 'win32':
		var Service = require('node-windows').Service; break;
	case 'linux':
		var Service = require('node-linux').Service; break;
	case 'darwin':
		var Service = require('node-mac').Service; break;
}

var args = process.argv.slice(2);
var services = [
    {
	name: 'appxConnector',
	port: 3014,
	script: require('path').join(__dirname,'appxConnector.js')
    },
    {
	name: 'appxMongoConnector',
	port: 3015,
	script: require('path').join(__dirname,'appxMongoConnector.js')
    }
];

/*
**Function to process arguments provided by the user. If no arguments or
**incorrect arguments are provided then we default install.
*/
function processArgs(){
    if (args.length > 0){
	if ( !isNaN(args[0]) ){
	    if (args.length > 2){
		argError();
	    }
	    for (var i = 0; i < args.length; i++){
		services[i].port = args[i];
		services[i].name = services[i].name.concat(args[i]);
	    }
	}else{
	    processServiceArg();
	}
    }else{
	console.log("No arguments recieved, using default settings for uninstall")
	for (var i = 0; i < services.length; i++){
	    services[i].name = services[i].name.concat(services[i].port);
	}
    }

}

/*
**Function to process service argument provided by the user. If incorrect
**argument is provided then we default install.
*/
function processServiceArg(){
    if (args.length > 3){
	argError();
    }else{
	switch (args[0]){
	     case "appx":
		 if (args.length === 2){
		     services[0].port = args[1];
		     services[0].name = services[0].name.concat(args[1]);
		 }
		 services.splice(1,1);
		 break;
	     case "mongo":
		 if (args.length === 2){
		     services[1].port = args[1];
		     services[1].name = services[1].name.concat(args[1]);
		 }
		 services.splice(0,1);
		 break;
	     default:
		 for (var i = 1; i < args.length; i++){
		     services[i-1].port = args[i];
		     services[i-1].name = services[i-1].name.concat(args[i]);
		 }
		 console.log("Both services are being uninstalled");
	}
    }
}

/*
**Function to throw error and abort program if to many arguments are provided with
**the command.
*/
function argError(){
    try{
	throw "Invalid Argument";
    }catch (ex){
	console.log("Invalid function call. Check the examples and try again.");
    }
}




function uninstallServices(mServices){
    //Create new service object
    var svc = new Service(mServices);

    //Listen for Uninstall event.
    svc.on('uninstall', function(){
	console.log('Uninstall complete.');
	console.log(mServices.name + ' service exists: ', svc.exists);
    });

    //Runs uninstall
    svc.uninstall();
}

processArgs();
for (var i = 0; i < services.length; i++){
    uninstallServices(services[i]);
}
