﻿//  *********************************************************
//  Configuration Section - Begin
//  *********************************************************

"use strict";
var mongoConnectorVersionStr = "5.4.5.16110211";
var mongoConnectorVersionNum = 50405.16110211;
var mongoStatus = "Running";
var connectorPort = process.env.APPX_MONGO_CONNECTOR_PORT;

var sslEnabled       = false;
var cryptoEnabled    = true; 
var sslPrivateKey    = "/etc/pki/tls/private/appx.key";
var sslCertificate   = "/etc/pki/tls/certs/appx.com.crt";
var sslCertAuthority = "/etc/pki/tls/certs/gd_bundle.crt";

var mongoDatabase    = "AppxDatabaseCache";
var mongoHost        = "localhost";
var mongoPort        = 27017;

//  *********************************************************
//  Configuration Section - End
//  *********************************************************

/************************************************************
    Library Imports - Begin
*************************************************************/
var StringLib   = require('string');
var GridStore   = require('mongodb').GridStore;
var MongoClient = require('mongodb').MongoClient;
var MongoServer = require('mongodb').Server;
var fs          = require('fs');
var WebSocket = require('ws').WebSocket;
var WebSocketServer = require('ws').Server;
var net = require('net');
var hexy = require('hexy');
var StringDecoder = require('string_decoder').StringDecoder;
var decoder = new StringDecoder('utf8');
var crypto = require('crypto');
var node_cryptojs = require('node-cryptojs-aes');
/************************************************************
    Library Imports - End
*************************************************************/

// Global variabls and Objects

var mongoClient = new MongoClient(new MongoServer(mongoHost, mongoPort));
var mongoUrl    = 'mongodb://'+mongoHost+':'+mongoPort+'/'+mongoDatabase+'?maxPoolSize=20';
var mongoCacheDb = null;

var initDatabase = function initDatabase() {
    try {
	mongoCacheDb.dropDatabase();
    } catch (e) {
	console.log("ERROR: failed to initialize mongoDatabase, err="+e);
	console.log(e.stack);
    }
};

mongoClient.connect(mongoUrl,function mongoClient_connectCallback(err, db) {
    mongoCacheDb = db;
    initDatabase();
    if (err){
	mongoStatus = "Error";
    }else{
	createWebSocket();
    }
});

var buildMongoSelection = function buildMongoSelection(gridRequest) {
    var selection = [];
    var result = {};
    if( gridRequest["_search"] == "true" ) {
	var filters = JSON.parse(gridRequest.filters);
	var groupOp = "$" + filters.groupOp.toLowerCase();
	var rules = filters.rules;
	
	for( var i=0; i<rules.length; i++ ) {
	    var rule = rules[i];
	    var field = rule.field;
	    var op = rule.op;
	    var data = rule.data;
	    var mongo = {};
	    var datatype = field.slice(-1);

	    switch(datatype) {
		 case "I": data = parseInt(data); break;
		 case "F": data = parseFloat(data); break;
		 case "B": data == "Y"; break;
	    }

	    mongo[field] = {};
	    
	    /* Test for uppercase letters in search string. If search string
	    ** contains uppercase then we do case sensitive search. Otherwise
	    ** searches are case insensitive.*/
	    if (/[A-Z]/.test(data) || !isNaN(data)){ 
		switch( op ) {
		     case 'eq': mongo[field]["$eq"] = data; break;
		     case 'ne': mongo[field]["$ne"] = data; break;
		     case 'lt': mongo[field]["$lt"] = data; break;
		     case 'gt': mongo[field]["$gt"] = data; break;
		     case 'le': mongo[field]["$lte"] = data; break;
		     case 'ge': mongo[field]["$gte"] = data; break;
		     case 'nu': mongo[field]["$eq"] = " "; break;
		     case 'nn': mongo[field]["$ne"] = " "; break;
		     case 'bw': mongo[field]["$regex"] = "^"+data; break;
		     case 'ew': mongo[field]["$regex"] = data+"$"; break;
		     case 'cn': mongo[field]["$regex"] = data; break;
		     case 'bn': mongo[field]["$regex"] = "^(?!"+data+").*"; break;
		     case 'en': mongo[field]["$regex"] = ".*(?<!"+data+")$"; break;
		     case 'nc': mongo[field]["$regex"] = "^((?!"+data+").)*$"; break;
		}
	    } else{
		switch( op ) {
		     case 'eq': mongo[field]["$regex"] = "^("+data+")$"; break;
		     case 'ne': mongo[field]["$regex"] = "^(?!"+data+")$"; break;
		     case 'lt': mongo[field]["$lt"] = data; break;
		     case 'gt': mongo[field]["$gt"] = data; break;
		     case 'le': mongo[field]["$lte"] = data; break;
		     case 'ge': mongo[field]["$gte"] = data; break;
		     case 'nu': mongo[field]["$eq"] = " "; break;
		     case 'nn': mongo[field]["$ne"] = " "; break;
		     case 'bw': mongo[field]["$regex"] = "^"+data; break;
		     case 'ew': mongo[field]["$regex"] = data+"$"; break;
		     case 'cn': mongo[field]["$regex"] = data; break;
		     case 'bn': mongo[field]["$regex"] = "^(?!"+data+").*"; break;
		     case 'en': mongo[field]["$regex"] = ".*(?<!"+data+")$"; break;
		     case 'nc': mongo[field]["$regex"] = "^((?!"+data+").)*$"; break;
		}
	    }

	    if ( mongo[field].hasOwnProperty("$regex") ){
		    mongo[field]["$options"] = "i";
	    }
	    selection.push(mongo);
	}
	result[groupOp] = selection;
    }
    return result;
}

var buildMongoSort = function buildMongoSort(gridRequest) {
    var sorter = {};
    var ord = 1;

    if( gridRequest.sidx && gridRequest.sidx.length > 0 ) {
	if( gridRequest.sord && gridRequest.sord == 'asc' )
	    sorter[gridRequest.sidx] = 1;
	else {
	    sorter[gridRequest.sidx] = -1;
	    ord = -1;
	}
    }
    /*If table was previously sorted we use that as a secondary sort*/
    if (Object.hasOwnProperty.call(gridRequest, "lastSortName[]") && gridRequest["lastSortName[]"].length >= 1){
	for (var i = gridRequest["lastSortName[]"].length - 1; i >=0 ; i--){
	    if (gridRequest["lastSortName[]"][i] !== gridRequest.sidx){
		if (gridRequest["lastSortOrder[]"][i] === "asc"){
		    sorter[gridRequest["lastSortName[]"][i]] = 1;
		}else{
		    sorter[gridRequest["lastSortName[]"][i]] = -1;
		}
	    }
	}
    }else {
	sorter["initialSort"] = ord;
    }

    return sorter;
}

var fetchtabledata = function fetchtabledata(message, res) {
    // This code is full of nested callbacks.  We have to open the database, then 
    // in that callback we have to open the collection, then inside that callback we have to
    // process the count function, then in that callback we get the requested records, and inside 
    // that callback we send the records to the client.  Whew, that's a lot of callbacks.

    // We are in the OPEN callback
    try {
	mongoCacheDb.collection(message.collection, function mongoCacheDb_collectionCallback(err, collection) {
	    // We are in the GET COLLECTION callback
	    var selection = buildMongoSelection(message);
	    var sorter = buildMongoSort(message);
	    var index = {};
	    var collist = {};
	    
	    if( message.collist ){
		collist = JSON.parse(message.collist);
	    }

	    collection.createIndex(sorter);
	    collection.find({$and:[{ "datalookupid": message.datalookupid },selection]}).count(function mongoCountCallback(err,totalcount){
		// We are in the COUNT results callback 
		collection.find({$and:[{ "datalookupid": message.datalookupid },selection]},collist).sort(sorter).skip((parseInt(message.page)-1)*parseInt(message.rows)).limit(parseInt(message.rows)).toArray(function mongoFindCallback(err, items) {
		    // We are in the GET RECORDS callback
		    var result = {};
		    result.records = parseInt(totalcount);
		    result.page = parseInt(message.page);
		    result.total = Math.ceil(parseInt(totalcount) / parseInt(message.rows));
		    var rowtmp = [];
		    if( items ) {
			for( var i=0; i < items.length; i++ ) {
			    rowtmp.push(items[i]);
			}
		    }
		    result["rows"] = rowtmp;
		    res.end(JSON.stringify(result));
		});
	    });
	});
    } catch (e) {
	console.log("ERROR: connecting to db");
	console.log(e);
	console.log(e.stack);
    }
};


function createWebSocket(){
    var CryptoJS = node_cryptojs.CryptoJS;
    var JsonFormatter = node_cryptojs.JsonFormatter;
    if (sslEnabled)
	var https = require('https');
    else
	var https = require('http');

    function processDataRequest(req, res) {
	var postData = "";
	var query;
	var url = require('url');
	var url_parts = url.parse(req.url, true);
	
	if (req.method === "POST"){
	    req.on("data", function req_onDataCallback(chunk){
		postData += chunk.toString();
	    });
	    req.on("end", function req_onEndCallback(){
		var qString = require("querystring");
		query = qString.parse(postData);
		getData();
		
	    });
	}else{
	    query = url_parts.query;
	    getData();
	}

	function getData(){
	    /*
	    **Function send HTTP error back to browser for display.
	    **
	    **@param mRes: Response to http request
	    */
	    function fileNotFound(mRes){
		/*If gridstore is not found, then we return a file not found
		**error*/
		res.writeHead(404,{'Content-Type': 'text/plain'});
		res.write('404 Not Found\n');
		res.end();
	    }
	    if( url_parts.pathname.indexOf('/getGridData') != -1 ) {
		res.writeHead(200, { 'Access-Control-Allow-Origin': '*', 'Content-Type': 'application/json' });	
		fetchtabledata( query, res );
	    }
	    else if( url_parts.pathname.indexOf("/getResource/") != -1 ) {
		var fileId = url_parts.pathname.substring(url_parts.pathname.indexOf("/getResource/")+13);
		var gridStore = new GridStore(mongoCacheDb, fileId, fileId, "r",{'root':'resource'});

		/*
		**Function wrapper to allow to be called again if we receive error on 
		**opening gridstore
		**
		**@param mCounter: Counter to make sure we don't create endless loop
		*/
		var ogs = function openGridStore(mCounter){
		    gridStore.open(function gridStore_openCallback(err,gridStore){
			if (err && (mCounter < 5)){
			    ogs(++mCounter);
			}
			if( gridStore ) {
			    var stream = gridStore.stream(true);
			    if ( gridStore.metadata.state < 8 ){			
				res.writeHead(200,{'Content-Type': 'image', 'Cache-Control': 'max-age=31536000'});
			    }else{
				res.writeHead(200,{'Content-Type': 'text', 'Cache-Control': 'max-age=31536000'});
			    }
			    stream.pipe(res);
			    gridStore.close(function gridStore_closeCallback(err, result){
			    });
			}else{
			    console.log("GridStore "+fileId+" not found");
			}
		    });
		}
		ogs(0);
	    }
	    else if( url_parts.pathname.indexOf("/getFile/") != -1 ) {
		/*If we pushed file into mongo to be displayed in the browser or use the
		**browser to save file to client*/
		var fileId = url_parts.pathname.substring(url_parts.pathname.indexOf("/getFile/")+9);
		var gridStore = new GridStore(mongoCacheDb, fileId, fileId, "r",{'root':'file'});
		/*
		**Function wrapper to allow to be called again if we receive error on 
		**opening gridstore
		**
		**@param mCounter: Counter to make sure we don't create endless loop
		*/
		var ogs = function openGridStore(mCounter){
		    gridStore.open(function gridStore_openCallback(err,gridStore){
			if (err && (mCounter < 5)){
			    ogs(++mCounter);
			}
			if( gridStore ) {
			    var stream = gridStore.stream(true);
			    var mime = require('mime');
			    var mimeType = mime.lookup(fileId);
			    res.writeHead(200,{'Access-Control-Allow-Origin': '*','Content-Type': mimeType, 'Cache-Control': 'no-cache, must-revalidate'});
			    stream.pipe(res);
			    /*We give a 5 second delay to make sure the file has finished 
			    **streaming to the client, then we remove the file from
			    **the gridstore and close the connection.*/
			    setTimeout(function setTimeoutCallback(){
				gridStore.unlink();
				gridStore.close(function gridStore_closeCallback(err, result){
				});
			    },5000);
			}else{
			    fileNotFound(res);
			}
		    });
		}
		ogs(0);
	    }
	    else if( url_parts.pathname.indexOf("/sendFile/") != -1 ) {
		/*If we pushed file into mongo to be uploaded to appx*/
		var fileId = url_parts.pathname.substring(url_parts.pathname.indexOf("/sendFile/")+10);
		var gridStore = new GridStore(mongoCacheDb, fileId, fileId, "r",{'root':'file'});
		/*
		**Function wrapper to allow to be called again if we receive error on 
		**opening gridstore
		**
		**@param mCounter: Counter to make sure we don't create endless loop
		*/
		var ogs = function openGridStore(mCounter){
		    gridStore.open(function gridStore_openCallback(err,gridStore){
			if (err && (mCounter < 5)){
			    ogs(++mCounter);
			}

			if( gridStore ) {
			    var stream = gridStore.stream(true);
			    var mime = require('mime');
			    var mimeType = mime.lookup(fileId);
			    res.writeHead(200,{'Access-Control-Allow-Origin': '*','Content-Type': mimeType, 'Cache-Control': 'max-age=5000, must-revalidate'});
			    stream.pipe(res);
			    setTimeout(function setTimeoutCallback(){
				gridStore.close(function gridStore_closeCallback(err, result){
				});
			    },50000);
			}else{
			    fileNotFound(res);
			}
		    });
		}
		ogs(0);
	    }
	}
    }

    // Configure the websocket
    if (sslEnabled) {
	var options = {
            key: fs.readFileSync(sslPrivateKey),
            cert: fs.readFileSync(sslCertificate),
            ca: fs.readFileSync(sslCertAuthority)
	};

	var app = https.createServer(options, function https_createServerCallback(req, res) {
	    processDataRequest(req,res);
	}).listen(connectorPort);
    }
    else {
	var app = https.createServer(function https_createServerCallback(req, res) {
	    processDataRequest(req,res);
	}).listen(connectorPort);
    }

    // start a websocket server
    var wss = new WebSocketServer({
	server: app
    });
}

